;This file is copyright (c) 1991, 1992 Informant Communications Group and the
;article author. The material here may be used in an application provided
;that this copyright/disclaimer information is kept in the original source
;file. The material presented here is provided "as is" and with no guarantee.
;Informant Communications Group/Paradox Informant assume no responsibility
;for the use or misuse of the material contained within.
;
;Contents        : procedure tbLockTables.l,
;                            tbView.u
;
;Source File     : QUERY.SC
;Author          : Dan Paolini
;                  DataStar International
;                  Larchmont Commons
;                  Suite 129
;                  3111 Route 38, #11
;                  Mount Laurel, NJ  08054
;                  (800) 742-3614
;
;Informant Issue : December 1992
;
;Description     : A generic query processor and locking routine
;
; Paradox Informant
; 10519 E. Stockton Blvd.
; Suite 142
; Elk Grove, CA  95624-9743
; Phone: (916) 686-6610
; Fax  : (916) 686-8497
; BBS  : (916) 686-4740

; ---------------------------------------------------------------------------
; The Paradox 4 Toolbox
; Copyright (c) 1992 - Daniel J. Paolini II - DataStar International
; ---------------------------------------------------------------------------
;       TITLE: quExecute.l
;     RETURNS: Logical true/false IF Query successful
; DESCRIPTION: Generic Query processor
PROC quExecute.l(                ; Generic Query Processor
         clear.l)                ; Should resultant table be cleared?
Private  qerror.l,               ; Error routine flag
         retval.l                ; Value to return
;Global  g.debug.l               ; Debugging flag
;        g.sysinfo.y             ; System Information DynArray
   qerror.l = false                       ; Initialize flag
   Do_It!                                 ; Main Errorproc checks for Query Run error
   IF qerror.l OR Window() <> "" THEN     ; Unsuccessful!
      msContinue.u("Query Error - " + Window(),79,999,999,"",3)
      retval.l = false                    ; Set return value to false
      IF IsAssigned(g.debug.l) AND g.debug.l THEN
         IF NOT IsAssigned(g.sysinfo.y) THEN
            SysInfo to g.sysinfo.y        ; Capture System Information
         ENDIF
         IF g.sysinfo.y["UIMode"] = "STANDARD" THEN
            AltSpace {Desktop}
            {SurfaceQueries}              ; Bring queries to top of Z order
         ENDIF
         DEBUG                            ; Invoke Debugger for Developer
      ENDIF
   ELSE
      IF clear.l THEN                     ; Clear Temporary Table
         IF Search(" " + Upper(Table()) + " ",
                   " ANSWER CHANGED DELETED INSERTED ") <> 0 THEN
            ClearImage
         ENDIF
      ENDIF
      IF NImages() >= 1 THEN              ; Might not be if querying Answer
         MoveTo 1                         ; 1 will always be Query, if on workspace
         WHILE ImageType() = "Query"      ; This cleans up the Query images
            ClearImage
         ENDWHILE
      ENDIF
      retval.l = true
   ENDIF
   Return retval.l
ENDPROC
; ---------------------------------------------------------------------------
; DESCRIPTION: Example of ERRORPROC evaluation of a Query error
; ---------------------------------------------------------------------------
;      CASE error.y["Code"] = 27 AND ImageType() = "Query"
;       AND error.y["Proc"] = "QUEXECUTE.L" :
;         qerror.l = true                  ; Set Query Error flag
;         retval.n = 1                     ; Return 1 = Skip over error command
; ---------------------------------------------------------------------------
;       TITLE: tbLockTables.l
;     RETURNS: Logical true/false IF lock successful
; DESCRIPTION: Generic lock table procedure using a lock string
;              lock.a must be created with QUADRUPLE backslashes, and the
;              quotes around the tables must be preceded by a backslash.
; ---------------------------------------------------------------------------
PROC tbLockTables.l(             ; Generic table locker
         lock.a)                 ; tables and locks to try
Private  lock.n,                 ; Lock increment counter
         retval.l                ; Value to return
   Message "Attempting to lock tables..."
   lock.n = 0                             ; Initialize counter
   oldretry.n = RetryPeriod(              ; Capture current Retry value
   SetRetryPeriod 0                       ; Turn off Retry
   WHILE true
      Execute "LOCK " + lock.a            ; using lock string
      IF retval THEN                      ; successful!
         retval.l = true
         QUITLOOP
      ENDIF
      IF lock.n < 5 THEN                  ; try 1 to 5 times in 5 seconds
         lock.n = lock.n + 1              ; Increment counter
         Sleep 1000                       ; Pause one second before trying again
         LOOP                             ; Try again
      ELSE                                ; Reached 5 attempts
         msWorking.u("Lock(s) Failed: " +ErrorMessage(),79,3,0)
         msConfirm.l("Do you want to try to lock the table(s) again?",
                      79,999,999,"",0,"~T~ry Again","~C~ancel",true)
         IF retval THEN
            lock.n = 0                    ; reset loop counter
            Message "Trying to Lock Tables Again..."
            LOOP                          ; Try again
         ELSE
            msWorking.u("C",31,1,2)       ; let's get out of here!
            retval.l = false              ; Unable to obtain locks
            QUITLOOP
         ENDIF
      ENDIF
   ENDWHILE
   SetRetryPeriod oldretry.n              ; Return to original value
   Message ""                             ; Clear persistent message
   Return retval.l                        ; True = Success
ENDPROC
; ---------------------------------------------------------------------------
;       TITLE: tbView.u
;     RETURNS: No Value
; DESCRIPTION: Checks for table on Workspace - Place Locks first
; ---------------------------------------------------------------------------
PROC tbView.u(                   ; Views table, locates it if on workspace
         table.a,                ; Table to view
         park.l)                 ; If true then move off screen
Private n,                       ; Transient image counter
        w                        ; Transient window handle
   IF NImages() > 0 THEN
      IF Upper(Table()) <> Upper(table.a) THEN
         FOR n From 1 to NImages()
            MoveTo n
            IF ImageType() <> "Query" THEN
               IF Upper(Table()) = Upper(table.a) THEN
                  QUITLOOP
               ENDIF
            ENDIF
         ENDFOR
         IF NOT Upper(Table()) = Upper(table.a) OR
            ImageType() = "Query" THEN
            View table.a
         ENDIF
      ENDIF
   ELSE
      View table.a
   ENDIF
   IF NOT IsAssigned(g.handles.y) THEN
      DynArray g.handles.y[]
   ENDIF
   Window Handle Image ImageNo() To w
   g.handles.y[table.a] = w
   IF park.l THEN
      utWindowPark.u(w)
   ENDIF
   Return
ENDPROC
